#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include "btree_mem.h"


#define MIN_DEGREE 2
#define MAX_LINE 1024

#ifdef DEBUG
#define DEBUG_BLOCK(x) x
#else
#define DEBUG_BLOCK(x)
#endif


////////////////////////////////////////////////////////////////////////////////
int
key_cmp(void * a, void * b)
   {
   if (*((int *) a) < *((int *) b))
      return -1;
   else if  (*((int *) a) == *((int *) b))
      return 0;
   return 1;
   }


////////////////////////////////////////////////////////////////////////////////
void
print_data(int depth, void const * const key, void * data)
   {
   for (int i = 0; i < depth; i++)
      printf("\t");
   printf("%d %s\n", *((int*)key), (char *) data);
   }

////////////////////////////////////////////////////////////////////////////////
void
wackify_data(int depth, void const * const key, void * data)
   {
   strcpy(data, "wack");
   }

////////////////////////////////////////////////////////////////////////////////
void
wackify_tree(struct btree_mem * bt)
   {
   btree_mem_walk_inorder(bt, wackify_data);
   }

////////////////////////////////////////////////////////////////////////////////
void
print_tree(struct btree_mem * bt)
   {
   printf("==================================\n");
   btree_mem_dump(bt, print_data);
   printf("==================================\n");
   }

////////////////////////////////////////////////////////////////////////////////
void
usage(char * progname)
   {
   printf("Usage: %s [mindegree]\n"
	  "mindegree is the minimum degree of the B-Tree to create. (As defined"
	  "by Cormen, et al., Introduction to Algorithms, Ch. 18.) The default"
	  "value is %d\n"
	  "\n"
	  "Data for the tree will be read from stdin.  The first field on the line"
	  "should be an integer and will be used as the key.  The remainder of the"
	  "line will be read as a single string that will be stored as the data."
	  , progname, MIN_DEGREE);
   }

////////////////////////////////////////////////////////////////////////////////
int
main(int argc, char ** argv)
   {
   int degree = MIN_DEGREE;

   if (argc > 1)
      {
      char * end;
      degree = strtol(argv[1], &end, 10);
      if (*end != '\0')
	 {
	 fprintf(stderr, "Degree must be an integer greater than or equal to %d\n", MIN_DEGREE);
	 usage(argv[0]);
	 exit(EXIT_FAILURE);
	 }
      }

   struct btree_mem * bt = btree_mem_new(degree, key_cmp);
   if (NULL == bt)
      {
      fprintf(stderr, "Unable to create new btree_mem\n");
      exit(EXIT_FAILURE);
      }

   DEBUG_BLOCK( print_tree(bt); );
   int minkey = INT_MAX;
   int maxkey = INT_MIN;
   int i;
   char s[MAX_LINE+1];
   while (1 == scanf("%d ", &i))
      {
      int * ip = malloc(sizeof(int));
      if (NULL == ip)
	 {
	 perror("malloc");
	 exit(EXIT_FAILURE);
	 }
      *ip = i;
      fgets(s, MAX_LINE, stdin);
      if (s[strlen(s)-1] != '\n')
	 {
	 fprintf(stderr, "input line exceeds maximum length of %d\n", MAX_LINE);
	 exit(EXIT_FAILURE);
	 }
      s[strlen(s)-1] = '\0';
      char * data = (s == NULL) ? NULL : strdup(s);


      if (*ip < minkey) minkey = *ip;
      if (*ip > maxkey) maxkey = *ip;

      printf("Inserting k %p, d %p\n", ip, data);
      if (NULL == btree_mem_insert(bt, ip, data))
	 {
	 fprintf(stderr, "btree_mem_insert failed\n");
	 exit(EXIT_FAILURE);
	 }
      DEBUG_BLOCK( print_tree(bt); )
      }
   //   wackify_tree(bt);
   print_tree(bt);


   printf("Deleting tree\n");
   int * k;
   char * d;
   for (int key = minkey; key <= maxkey; key++)
      {
      if (NULL == btree_mem_remove(bt, &key, (void *) &k, (void *) &d))
	 {
	 printf("not found!\n");
	 }
      else
	 {
	 if (NULL == k)
	    printf("k is NULL\n");
	 else
	    free(k);

	 if (NULL == d)
	    printf("d is NULL\n");
	 else
	    free(d);
	 }
      }
   print_tree(bt);
   
   btree_mem_delete(bt);

   exit(EXIT_SUCCESS);
   }
